//
//	Base filename LCDdisplay.C
//	
//	Module:	LCD Display routines
//	Model:		1
//	Version:	1
//	
//	Revision History:
//	-----------------
//                                    
//	Author:			Date:	   Version:	Description:
//	-------			-----	   --------	------------
//	T. Snoblen		11/25/96     1.0	Original implementation.

//
//	Copyright Tom Snoblen 1999
//==============================================================================

#include <reg51.h>                /* special function register declarations   */
                                  /* for the intended 8051 derivative         */

#include <stdio.h>                /* prototype declarations for I/O functions */

# include "lcddisp.h"  

///////////////////////////////////////////////////////////////////////////////////////////////
// write_display (char ch,char reg)
//    ch = char to be displayed on lcd or register data
//    reg = mode WRITEREG ,  WRITEDATA
//  This routine writes data to the lcd display
//
//
//////////////////////////////////////////////////////////////////////////////////////////////   
void write_display(char ch,char reg){
   int i;
   display_busy();
   P2 &= REGRWMASK ;			// reset the control lines
   P2 |= reg ;          		// set the operation
   P0 = ch   ; 				//put the data on the bus
   P2 |= ENABLE_DISPLAY ; 		// doit
   for (i = 0 ; i<DISPLAY_DELAY ; i++){}//delay
   P2 &=ENABLE_MASK;    		// reset the enable
   for (i = 0 ; i<DISPLAY_DELAY ; i++){}//delay
   P2 &= REGRWMASK ;			// reset the control lines
   P0 = 0 ;//low power
}


void display_busy(){
   int i;

   P2 |= READREG;
   
   do{
      
      P2 |= ENABLE_DISPLAY ; 		    //make it inactive
      for (i = 0 ; i<DISPLAY_DELAY ; i++){}//delay
      P2 &= ~ENABLE_DISPLAY ;		    // latch the data
      for (i = 0 ; i<DISPLAY_DELAY ; i++){}//delay
   }while (P0 & 0x80) ;
   P2 &= REGRWMASK;		    // reset the control lines
   for (i = 0 ; i<DISPLAY_DELAY ; i++){}//delay
   

}
///////////////////////////////////////////////////////////////////////////////////////////////
// 
//   display_string (char* ch,int nLoc)
//   ch = string to be displayed
//   nLoc = address of first char
//
//   This routine displays a string at a location chosen by the nLOC
//
//////////////////////////////////////////////////////////////////////////////////////////////   
void display_string (char* ch,int nLoc){
   int i;
   i=0;
    write_display(DDRAM+nLoc,WRITEREG); //position next char.
   while (ch[i]>0){                     // loop till all char sent
      write_display(ch[i],WRITEDATA);   // write the char to the LCD
      i++;                              // point at next char
      if (i==20)break;                  // break if to many char
   }

}


///////////////////////////////////////////////////////////////////////////////////////////////
// 
//   display_setup ()
//  
//
//   This routine configurs the LCD display
//
//////////////////////////////////////////////////////////////////////////////////////////////
void display_setup (){
   

   write_display(FUNCTION,WRITEREG);
   write_display(SHIFT,WRITEREG);
   write_display(MODE,WRITEREG);
   write_display(ON,WRITEREG);
   write_display(CLEAR,WRITEREG);
   // end of init code
   
}
 
