//
//	Base filename:.Charger.C
//	
//	Project:	CitiVan Charge Computer
//	Model:		1
//	Version:	1
//	
//	Revision History:
//	-----------------
//                 
                   
//	Author:			Date:	   Version:	Description:
//	-------			-----	   --------	------------
//	T. Snoblen		3/31/99     1.0	Original implementation.

//
//	Copyright Tom Snoblen 1999
//==============================================================================
#pragma DEBUG OBJECTEXTEND CODE   /* pragma lines can contain state C51 */

#include "charger.h"
#include <reg51.h>                /* special function register declarations   */
                                  /* for the intended 8051 derivative         */

#include <stdio.h>                /* prototype declarations for I/O functions */

#include "lcddisp.h"      
                                  /* command line directives                  */




// /////////////////////////////////////////////////////////////////////////////////////
// Globals
// external RAM Base
unsigned int	nTic=0               ;  // System tic counter
unsigned char   ucPhaseCount=0        ;  // current  phasposition
unsigned int	unPWMcount=MAXTIME        ;  // pwm value
unsigned int	unADphase = 00	     ;  // current phase of the atod conversion


// P2 defs
#define LEDON		0x01
#define LEDOFF	        0xfe 

#define ADCLOCKON  	0x10	//mask to trun on AtoD clock
#define ADCLOCKOFF      0xef    //mask to trun on AtoD clock
#define ADSELON         0x20    // select atoD 
#define ADSELOFF        0xdf    // deselect atoD start conversion
#define DOUT            0x40    // data out mask  
#define PWMON		0x80
#define PWMOFF          0x7f





// ROM data tables
code unsigned int    anCalData[6]= {0x870c,  0x8708, 0x8704};
///////////////////////////////////////////////////////////////////////////////////////////////
// ZeroCross
//  
//  interrupts on the sin wave near the zero crossing point
//  reset the nPhaseCounter
//
//////////////////////////////////////////////////////////////////////////////////////////////    
void ZeroCross (void)  interrupt    0 using 3{
   
   ucPhaseCount=1;                       // reset the phase
   TH0=TIMERHIGH;
   TL0=TIMERLOW;  			//help remove jitter
   if(0 == unPWMcount)  P2  |=PWMON ;    // turn on pulse for first phase
         
   else     P2 &= PWMOFF ; 
//  nDelay = DELAY  ;  
 
}





///////////////////////////////////////////////////////////////////////////////////////////////
// timer0
//  interrupt 1 using int 2
//
// This routine is triger by the internal timer and all processes that
// need updating on a regular bias shoud be included in this routine.
//
//
//////////////////////////////////////////////////////////////////////////////////////////////   
void timer0 (void)  interrupt    1 using 2{
   
// Tic timer
   nTic++;   					// update global tic counter
   TL0=TIMERLOW; 			// Reset the internal timer  for 461 counts
   TH0=TIMERHIGH  ;

   if ( nTic & LEDTOGGLE){
      P2 |= LEDON ;                    		// turn on led
   }else P2 &= LEDOFF ;         		// turn off led	

   
      
 //  if (nDelay>=0) nDelay--;
 //  else ucPhaseCount=0;

   

   if (unPWMcount < MAXTIME){ 
      

 //  if(ucPhaseCount <  unPWMcount) 	// turn of time
 //     P2 &= PWMOFF ;
     if(ucPhaseCount == unPWMcount)         	// turn on pulse for first phase
           P2 |=PWMON ;  
     
     if(ucPhaseCount == unPWMcount+TIME120)	// turn on pulse for second phase
          P2 |=PWMON  ;  
     if(ucPhaseCount == unPWMcount+PWMWITH) 	// turn of time
        P2 &= PWMOFF ; 
     if(ucPhaseCount >= unPWMcount+PWMWITH+TIME120) // turn of time
        P2 &= PWMOFF ;        			// update the pwm phase counter
     if (ucPhaseCount < 3*MAXTIME)ucPhaseCount++;
   }else P2 &= PWMOFF ;
  
}



///////////////////////////////////////////////////////////////////////////////////////////////
// 
//   Main ()
//  
//
//   main init. the hardware
//   Then go into endless loop that
//	includes all neccassary functions
//
//////////////////////////////////////////////////////////////////////////////////////////////
void main (void)  {     // execution starts here after stack init
   int i ;
   unsigned int unPercent=0; 
   char 	szOutput[21];   // output data
   unsigned char ucOld ;// hold output string 

   bit 		bSw1on	 ;	// true if sw1 has gone to the true state
   bit 		bSw2on	 ;	// true if sw2 has gone to the true state
  
   for (i = 0 ; i<10000 ; i++){}//delay

   ucOld= 0;
   TCON  |= 0x01 ;	// set for edgtriggered
   SCON  = 0x50;        // SCON: mode 1, 8-bit UART, enable rcvr    
   TMOD |= 0x21;        // TMOD: timer 1, mode 2, 8-bit reload  
		        //       timer 0, mode 1, 16 bit timer	    
   TH1   = 0xfd;        // TH1:  reload value for 9600 baud         
   TR1   = 1;           // TR1:  timer 1 run
   TR0   = 1;	        // Start tic counter
   IT0   = 1;           // set to rdge trigger ext int
   P0    = 0 ;          // zero out display data                     
   P1    = 0xff;        // input port 
   TI    = 1 ;          // prime serial transmite
   P2	 = 0;           // zero the output port	
   IE    = 0x83; 	// int controll timer0 and ex0  on
                        // initialize display
   

   bSw1on=FALSE;
   bSw2on=FALSE;
   
   display_setup(); 
  #if RS232
   printf(VER); // send version number to serial port
  #endif
 
   display_string("Manual Mode",64);  
   //sprintf(szOutput,"PWM   %d %% ", unPWMcount);
  // display_string(szOutput,0x0);
   // here is the main loop
   while (1) {                     /* An embedded program does not stop and     */
      sprintf(szOutput,"PWM   %u %%   ", (MAXTIME-unPWMcount)*100/MAXTIME);
      display_string(szOutput,0x0);

#if RS232
      // rs232 monitor code       
      printf("%d, %u%%, \n",
          nTic,(int)((MAXTIME-unPWMcount)*100/MAXTIME));
 #endif
    
      if(P1 & SWITCH1){ // if switch one is pressed
	 bSw1on=TRUE;

      }else{
         if (bSw1on==TRUE) {
	    bSw1on=FALSE;
	    if (unPercent < MAXTIME ) unPercent =unPercent+PERCENTINC ;	
	}
      }

      if(P1 & SWITCH2){ // if switch one is pressed
	 bSw2on=TRUE;

      }else{
         if (bSw2on) {
	    bSw2on=FALSE;
	    // range check
            if (unPercent < PERCENTINC )    unPercent =0;
            else  unPercent = unPercent-PERCENTINC  ;	
	}
      }
      unPWMcount=MAXTIME-unPercent;// percent conversion
   }
}
